@@ -16,7 +16,7 @@ void ElectronCloud_track_local_particle(
 
     const double x_shift = ElectronCloudData_get_x_shift(el);
     const double y_shift = ElectronCloudData_get_y_shift(el);
-    const double tau_shift = ElectronCloudData_get_tau_shift(el);
+    const double zeta_shift = ElectronCloudData_get_zeta_shift(el);
     TriCubicInterpolatedFieldMapData fmap = ElectronCloudData_getp_fieldmap(el);
     //start_per_particle_block (part0->part)
     const double x = LocalParticle_get_x(part);
@@ -25,22 +25,20 @@ void ElectronCloud_track_local_particle(
 
     double const beta0 = LocalParticle_get_beta0(part);
 
-    double const tau = zeta / beta0;
-
     double dphi_dx=0;
     double dphi_dy=0;
-    double dphi_dtau=0;
+    double dphi_dzeta=0;
 
     if( TriCubicInterpolatedFieldMap_interpolate_grad(fmap,
-        x - x_shift, y - y_shift, tau - tau_shift,
-        &dphi_dx, &dphi_dy, &dphi_dtau)
+        x - x_shift, y - y_shift, zeta - zeta_shift,
+        &dphi_dx, &dphi_dy, &dphi_dzeta)
       ){
           LocalParticle_set_state(part, XF_OUTSIDE_INTERPOL); // Stop tracking particle if it escapes the interpolation grid.
       }
 
     const double px_kick = - dphi_dx * length - ElectronCloudData_get_dipolar_px_kick(el);
     const double py_kick = - dphi_dy * length - ElectronCloudData_get_dipolar_py_kick(el);
-    const double ptau_kick = - dphi_dtau * length - ElectronCloudData_get_dipolar_ptau_kick(el);
+    const double pzeta_kick = - dphi_dzeta * length - ElectronCloudData_get_dipolar_pzeta_kick(el);
 
     // TODO: implement kicks for particles with different charge and or mass
     LocalParticle_add_to_px(part, px_kick);
@@ -48,7 +46,7 @@ void ElectronCloud_track_local_particle(
 
     double const q = LocalParticle_get_q0(part);
     double const p0c = LocalParticle_get_p0c(part);
-    double const energy_change = q * (p0c * ptau_kick);
+    double const energy_change = q * ( ( beta0 * p0c ) * pzeta_kick);
     LocalParticle_add_to_energy(part, energy_change, 1);
 
     //end_per_particle_block
