@@ -7,12 +7,10 @@ from scipy.special import  erfinv
 from ._madpoint import MadPoint
 import xfields as xf
 
-
-
 def install_beambeam_elements_in_lines(line_b1, line_b4, ip_names,
             harmonic_number, bunch_spacing_buckets,
             num_long_range_encounters_per_side, num_slices_head_on,
-            sigmaz_m):
+            sigmaz_m, delay_at_ips_slots=None):
 
     keep_columns = ['beam', 'other_beam', 'ip_name', 'elementName', 'other_elementName', 'label',
             'self_particle_charge', 'self_relativistic_beta', 'self_frac_of_bunch',
@@ -52,6 +50,10 @@ def install_beambeam_elements_in_lines(line_b1, line_b4, ip_names,
     else:
         bb_df_b2 = None
 
+    if delay_at_ips_slots is not None:
+        _compute_delays(bb_df_b1, bb_df_b2, delay_at_ips_slots, ip_names,
+                        harmonic_number, bunch_spacing_buckets)
+
     return bb_df_b1, bb_df_b2
 
 def configure_beam_beam_elements(bb_df_cw, bb_df_acw, line_cw, line_acw,
@@ -75,7 +77,7 @@ def configure_beam_beam_elements(bb_df_cw, bb_df_acw, line_cw, line_acw,
         if bb_df is None:
             continue
 
-        twiss = line.twiss()
+        twiss = line.twiss(reverse=False) # Ignore twiss_default for reverse
         if orientation == 'acw':
             tw_acw = twiss
             twiss = twiss.reverse()
@@ -84,7 +86,7 @@ def configure_beam_beam_elements(bb_df_cw, bb_df_acw, line_cw, line_acw,
         surveys = {}
 
         for ip_name in ip_names:
-            sv_ip = line.survey(element0=ip_name)
+            sv_ip = line.survey(element0=ip_name, reverse=False) # Ignore twiss_default for reverse
             if orientation == 'acw':
                 sv_ip = sv_ip.reverse()
             surveys[ip_name] = sv_ip
@@ -160,6 +162,11 @@ def install_dummy_bb_lenses(bb_df, line):
     for iipp in ip_names:
         s_ips[iipp] = line.get_s_position(iipp)
 
+    s_insertions = []
+    for nn in bb_df.index:
+        s_insertions.append(s_ips[bb_df.loc[nn, 'ip_name']] + bb_df.loc[nn, 'atPosition'])
+    line.cut_at_s(s_insertions)
+
     for nn in bb_df.index:
         print(f'Insert: {nn}     ', end='\r', flush=True)
         ll = bb_df.loc[nn, 'label']
@@ -716,7 +723,7 @@ def setup_beam_beam_in_line(
 
 def measure_crabbing(line, bb_df, reverse):
 
-    tw = line.twiss()
+    tw = line.twiss(reverse=False) # Ignore twiss_default for reverse
     if reverse:
         tw = tw.reverse()
 
@@ -728,7 +735,8 @@ def measure_crabbing(line, bb_df, reverse):
             if reverse:
                 zeta0 = -zeta0 # LHC convention
             tw4d_crab = line.twiss(method='4d',zeta0=zeta0,
-                                   freeze_longitudinal=True)
+                                   freeze_longitudinal=True,
+                                   reverse=False) # Ignore twiss_default for reverse
             if reverse:
                 tw4d_crab = tw4d_crab.reverse()
             ii = np.where(np.array(tw.name) == nn)[0][0]
@@ -740,5 +748,72 @@ def measure_crabbing(line, bb_df, reverse):
             for coord in ['x', 'px', 'y', 'py']:
                 bb_df.loc[nn, f'self_{coord}_crab'] = 0.0
 
+def _compute_delays(bb_df_cw, bb_df_acw, delay_at_ips_slots, ip_names,
+                    harmonic_number, bunch_spacing_buckets):
 
+    ring_length_in_slots = harmonic_number / bunch_spacing_buckets
 
+    for orientation, bbdf  in zip(['clockwise', 'anticlockwise'],
+                                  [bb_df_cw, bb_df_acw]):
+
+        if orientation == 'clockwise':
+            delay_at_ips_dict = {iipp: dd
+                                for iipp, dd in zip(ip_names, delay_at_ips_slots)}
+        elif orientation == 'anticlockwise':
+            delay_at_ips_dict = {iipp: np.mod(ring_length_in_slots - dd, ring_length_in_slots)
+                                for iipp, dd in zip(ip_names, delay_at_ips_slots)}
+        else:
+            raise ValueError('?!')
+
+        delay_in_slots = []
+
+        for nn in bbdf.index.values:
+            ip_name = bbdf.loc[nn, 'ip_name']
+            this_delay = delay_at_ips_dict[ip_name]
+
+            if nn.startswith('bb_lr.'):
+                if orientation == 'clockwise':
+                    this_delay += bbdf.loc[nn, 'identifier']
+                elif orientation == 'anticlockwise':
+                    this_delay -= bbdf.loc[nn, 'identifier']
+                else:
+                    raise ValueError('?!')
+
+            delay_in_slots.append(int(this_delay))
+
+        bbdf['delay_in_slots'] = delay_in_slots
+
+def apply_filling_pattern(collider, filling_pattern_cw, filling_pattern_acw,
+                          i_bunch_cw, i_bunch_acw):
+
+    dframes = collider._bb_config['dataframes']
+
+    ring_length_in_slots = int(collider._bb_config['harmonic_number']
+                            / collider._bb_config['bunch_spacing_buckets'])
+
+    for orientation_self in ['clockwise', 'anticlockwise']:
+
+        if orientation_self == 'clockwise':
+            filling_pattern_self = np.array(filling_pattern_cw, dtype=int)
+            filling_pattern_other = np.array(filling_pattern_acw, dtype=int)
+            i_bunch_self = i_bunch_cw
+        else:
+            filling_pattern_self = np.array(filling_pattern_acw)
+            filling_pattern_other = np.array(filling_pattern_cw)
+            i_bunch_self = i_bunch_acw
+
+        assert set(list(filling_pattern_self)).issubset({0, 1})
+        assert set(list(filling_pattern_other)).issubset({0, 1})
+
+        assert filling_pattern_self[i_bunch_self] == 1, "Selected bunch is not in the filling scheme"
+
+        temp_df = dframes[orientation_self].loc[:, ['delay_in_slots', 'ip_name']].copy()
+        temp_df['partner_bunch_index'] = dframes[orientation_self]['delay_in_slots'] + i_bunch_self
+        temp_df['partner_bunch_index'] = np.mod(temp_df['partner_bunch_index'], ring_length_in_slots)
+        temp_df['is_active'] = filling_pattern_other[temp_df['partner_bunch_index']] == 1
+
+        for nn, state in temp_df['is_active'].items():
+            if state:
+                collider.vars[nn + '_scale_strength'] = collider.vars['beambeam_scale']
+            else:
+                collider.vars[nn + '_scale_strength'] = 0
