@@ -6,10 +6,9 @@
 import numpy as np
 
 import xobjects as xo
-import xpart as xp
 import xtrack as xt
 
-from ..solvers.fftsolvers import FFTSolver3D, FFTSolver2p5D
+from ..solvers.fftsolvers import FFTSolver3D, FFTSolver2p5D, FFTSolver2p5DAveraged
 from ..general import _pkg_root
 
 _TriLinearInterpolatedFielmap_kernels = {
@@ -29,7 +28,7 @@ _TriLinearInterpolatedFielmap_kernels = {
     'p2m_rectmesh3d_xparticles': xo.Kernel(
         args=[
             xo.Arg(xo.Int32,   pointer=False, name='nparticles'),
-            xo.Arg(xp.Particles, pointer=False, name='particles'),
+            xo.Arg(xt.Particles, pointer=False, name='particles'),
             xo.Arg(xo.Float64, pointer=False, name='x0'),
             xo.Arg(xo.Float64, pointer=False, name='y0'),
             xo.Arg(xo.Float64, pointer=False, name='z0'),
@@ -158,12 +157,13 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
 
     _extra_c_sources = [
         _pkg_root.joinpath('headers/constants.h'),
+        xt.general._pkg_root.joinpath('headers/atomicadd.h'),
         _pkg_root.joinpath('fieldmaps/interpolated_src/central_diff.h'),
         _pkg_root.joinpath('fieldmaps/interpolated_src/linear_interpolators.h'),
         _pkg_root.joinpath('fieldmaps/interpolated_src/charge_deposition.h'),
         ]
 
-    _depends_on = [xp.Particles]
+    _depends_on = [xt.Particles]
 
     _kernels = _TriLinearInterpolatedFielmap_kernels
 
@@ -215,7 +215,10 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
                  dphi_dy = nelem,
                  dphi_dz = nelem)
 
-        self.compile_kernels(only_if_needed=True)
+        self.compile_kernels(
+            only_if_needed=True,
+            extra_compile_args=[f"-I{xt.__path__[0]}"],
+        )
 
         if isinstance(solver, str):
             self.solver = self.generate_solver(solver, fftplan)
@@ -237,7 +240,6 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
     def _assert_updatable(self):
         assert self.updatable, 'This FieldMap is not updatable!'
 
-    #@profile
     def get_values_at_points(self,
             x, y, z,
             return_rho=True,
@@ -304,7 +306,7 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
                     offsets_mesh_quantities=pos_in_buffer_of_maps_to_interp,
                     particles_quantities=buffer_out)
 
-        # Split buffer 
+        # Split buffer
         particles_quantities = [buffer_out[ii*len(x):(ii+1)*len(x)]
                                         for ii in range(nmaps_to_interp)]
 
@@ -382,6 +384,11 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
                     grid1d_offset=self._xobject.rho._offset
                                  +self._xobject.rho._data_offset)
 
+        if hasattr(self, '_average_transverse_distribution'):
+            raise NotImplementedError(
+                '`_average_transverse_distribution` has been removed, '
+                'use `solver=FFTSolver2p5DAveraged` instead')
+
         if update_phi:
             self.update_phi_from_rho(solver=solver)
 
@@ -414,7 +421,7 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
         updated.
 
         Args:
-            rho (float64 array): Potential at the grid points.
+            phi (float64 array): Potential at the grid points.
             reset (bool): If ``True`` the stored potential is overwritten
                 with the provided one. If ``False`` the provided potential
                 is added to the stored one. The default is ``True``.
@@ -523,6 +530,14 @@ class TriLinearInterpolatedFieldMap(xo.HybridClass):
                     nx=self.nx, ny=self.ny, nz=self.nz,
                     context=self._buffer.context,
                     fftplan=fftplan)
+        elif solver == 'FFTSolver2p5DAveraged':
+            solver = FFTSolver2p5DAveraged(
+                    dx=self.dx*scale_dx,
+                    dy=self.dy*scale_dy,
+                    dz=self.dz*scale_dz,
+                    nx=self.nx, ny=self.ny, nz=self.nz,
+                    context=self._buffer.context,
+                    fftplan=fftplan)
         else:
             raise ValueError(f'solver name {solver} not recognized')
 
